@@ -16,4 +16,5 @@ from cacab.plot.main import \
     annotated_axline, \
     PointSelector, \
     freeze_axes, \
-    setup_orsted
+    setup_orsted, \
+    specify_zoom_ax
