@@ -8,7 +8,9 @@ import matplotlib.dates
 from matplotlib.colors import ListedColormap
 from matplotlib.widgets import RectangleSelector
 from matplotlib.transforms import Bbox, TransformedBbox
+import matplotlib.patches as pch
 import matplotlib.lines
+from matplotlib.artist import Artist
 import numpy as np
 from scipy.ndimage import gaussian_filter
 import colorsys
@@ -286,7 +288,7 @@ def annotated_axline(ax, value, text, ax_type: str = 'h', text_loc=0.5, axline_k
     textcoords = {"v":("data", "axes fraction"), "h":("axes fraction", "data")}.get(ax_type)
     if rotation is None:
         rotation = {"h": 0, "v": 90}.get(ax_type)
-    ax.annotate(text=text, xy=(0, 0), xytext=xytext, textcoords=textcoords, va='center', ha='center', rotation=rotation)
+    ax.annotate(text=text, xy=(0.5, 0.5), xytext=xytext, xycoords=("axes fraction", "axes fraction"), textcoords=textcoords, va='center', ha='center', rotation=rotation)
 
     bbox_low = {
         "h": TransformedBbox(Bbox([[0, 0], [text_loc - clear_line_width / 2, 1]]), transform=ax.transAxes),
@@ -511,7 +513,7 @@ def freeze_axes(ax):
     xlim_base = ax.get_xlim()
     ylim_base = ax.get_ylim()
     ax.callbacks.connect("xlim_changed", lambda _, ax_i=ax, xlim_i=xlim_base: ax_i.set_xlim(xlim_i) if ax_i.get_xlim() != xlim_i else None)
-    ax.callbacks.connect("ylim_changed", lambda _, ax_j=ax, ylim_j=ylim_base: ax_j.set_ylim(ylim_j) if ax_j.get_xlim() != ylim_j else None)
+    ax.callbacks.connect("ylim_changed", lambda _, ax_j=ax, ylim_j=ylim_base: ax_j.set_ylim(ylim_j) if ax_j.get_ylim() != ylim_j else None)
 
 def setup_orsted():
     matplotlib.use("Qt5Agg")
@@ -522,10 +524,82 @@ def setup_orsted():
     })
 
 
+def specify_zoom_ax(ax_zoom: plt.axes, ax_main, c, ltrb_transform=None, rect_kwargs=None, connection_path_kwargs=None, update=True):
+    rect_kwargs = rect_kwargs if rect_kwargs is not None else dict()
+    connection_path_kwargs = connection_path_kwargs if connection_path_kwargs is not None else dict()
+    rect_base_kwargs = dict(lw=2, edgecolor=c, facecolor=c, alpha=0.3, zorder=99)
+    rect_zoom_kwargs = dict(lw=4, edgecolor=c, facecolor=c, alpha=0.3, zorder=-100)
+    if ltrb_transform:
+        rect_kwargs.update(transform=ltrb_transform)
+    rect_base_kwargs.update(rect_kwargs)
+    rect_zoom_kwargs.update(rect_kwargs)
+
+    connection_path_base_kwargs = dict(color=c, alpha=0.2, annotation_clip=False, zorder=100)
+    connection_path_base_kwargs.update(connection_path_kwargs)
+
+    for spine in ax_zoom.spines.values():
+        spine.set_edgecolor(c)
+
+    conn_patches = []
+    main_box_patch = None
+    zoom_box_patch = None
+
+    def set_zoom(ax_event):
+        left_top_right_bottom = (
+            ax_zoom.get_xlim()[0], ax_zoom.get_ylim()[1], ax_zoom.get_xlim()[1], ax_zoom.get_ylim()[0])
+        left, top, right, bottom = [
+            matplotlib.dates.date2num(v.to_datetime64()) if isinstance(v, pd.Timestamp) else v
+            for v in left_top_right_bottom
+        ]
+        width = (right - left)
+        height = (top - bottom)
+        center = (left, bottom)
+
+        nonlocal main_box_patch
+        if main_box_patch is None:
+            main_box_patch = pch.Rectangle(center, width, height, **rect_base_kwargs)
+            ax_main.add_artist(main_box_patch)
+        else:
+            main_box_patch.set_xy(center)
+            main_box_patch.set_width(width)
+            main_box_patch.set_height(height)
+
+        nonlocal zoom_box_patch
+        if zoom_box_patch is None:
+            zoom_box_patch = pch.Rectangle(center, width, height, **rect_base_kwargs)
+            ax_zoom.add_artist(zoom_box_patch)
+        else:
+            zoom_box_patch.set_xy(center)
+            zoom_box_patch.set_width(width)
+            zoom_box_patch.set_height(height)
+
+        four_pair_points = [
+            ((left, top), (0, 1)),
+            ((right, top), (1, 1)),
+            ((right, bottom), (1, 0)),
+            ((left, bottom), (0, 0)),
+        ]
+
+        if len(conn_patches) == 0:
+            for pair_points in four_pair_points:
+                conn_patch = pch.ConnectionPatch(pair_points[0], pair_points[1], "data", "axes fraction", ax_main, ax_zoom,
+                                                 **connection_path_base_kwargs)
+                conn_patches.append(conn_patch)
+                ax_main.get_figure().add_artist(conn_patch)
+        else:
+            for i in range(4):
+                conn_patches[i].xy1 = four_pair_points[i][0]
+
+    if update:
+        ax_zoom.callbacks.connect("xlim_changed", set_zoom)
+        ax_zoom.callbacks.connect("ylim_changed", set_zoom)
+    set_zoom(None)
+
+
 if __name__ == '__main__':
     plt.close('all')
     matplotlib.use('Qt5Agg')
-    selector_test = True
+    selector_test = False
     if selector_test:
         import pandas as pd
 
@@ -596,3 +670,28 @@ if __name__ == '__main__':
         add_ax_distribution(fig, ax, x1, y1, c=defaultcolors[0], ax_right_len=0.03, y_sigma=0.05)
         add_ax_distribution(fig, ax, x2, y2, c=defaultcolors[1], ax_right_len=0.03, y_sigma=0.05)
         plt.show()
+
+    set_zoom_ax_test = False
+    if set_zoom_ax_test:
+        import numpy as np
+        setup_orsted()
+        fig = plt.figure(figsize=(10, 5))
+        fig.gridspec = gs.GridSpec(1, 2)
+        axs = [fig.add_subplot(fig.gridspec[0, i]) for i in range(2)]
+        n = 10000
+        x = np.random.normal(0, 1, n)
+        y = np.random.normal(0, 1, n)
+        axs[0].scatter(x, y, c=defaultcolors[0], marker='.')
+        axs[1].scatter(x, y, c=defaultcolors[0], marker='.')
+        axs[1].set_xlim([-0.3, 0.3])
+        axs[1].set_ylim([-0.3, 0.3])
+        set_zoom_ax(axs[1], axs[0], c=defaultcolors[1])
+        plt.show()
+
+    annotated_ax_test = True
+    if annotated_ax_test:
+        fig = plt.figure()
+        ax = fig.gca()
+        ax.plot([pd.Timestamp("2022-01-01"), pd.Timestamp("2022-01-02")], [0, 1])
+        annotated_axline(ax, pd.Timestamp("2022-01-01 12:00"), "center", "v", text_loc=0.1, axline_kwargs={"c":"k", "ls":"--"})
+        plt.show()
